// Generated by CoffeeScript 1.10.0
(function() {
  var ArgumentsValidator, ChatServiceError, FastMap, Promise, _, check, possiblyCallback,
    slice = [].slice;

  ChatServiceError = require('./ChatServiceError');

  FastMap = require('collections/fast-map');

  Promise = require('bluebird');

  _ = require('lodash');

  check = require('check-types');

  possiblyCallback = require('./utils').possiblyCallback;

  ArgumentsValidator = (function() {
    function ArgumentsValidator(server) {
      var fn, name, ref;
      this.server = server;
      this.checkers = new FastMap;
      this.directMessagesChecker = this.server.directMessagesChecker;
      this.roomMessagesChecker = this.server.roomMessagesChecker;
      this.customCheckers = {
        directMessage: [null, this.directMessagesChecker],
        roomMessage: [null, this.roomMessagesChecker]
      };
      ref = this.server.userCommands;
      for (name in ref) {
        fn = ref[name];
        this.checkers.set(name, this[name]());
      }
    }

    ArgumentsValidator.prototype.checkArguments = function() {
      var args, cb, name, ref;
      name = arguments[0], args = 2 <= arguments.length ? slice.call(arguments, 1) : [];
      ref = possiblyCallback(args), args = ref[0], cb = ref[1];
      return Promise["try"]((function(_this) {
        return function() {
          var checkers, customCheckers, error;
          checkers = _this.checkers.get(name);
          if (!checkers) {
            error = new ChatServiceError('noCommand', name);
            return Promise.reject(error);
          }
          error = _this.checkTypes(checkers, args);
          if (error) {
            return Promise.reject(error);
          }
          customCheckers = _this.customCheckers[name] || [];
          return Promise.each(customCheckers, function(checker, idx) {
            if (checker) {
              return Promise.fromCallback(function(fn) {
                return checker(args[idx], fn);
              });
            }
          });
        };
      })(this)).asCallback(cb);
    };

    ArgumentsValidator.prototype.getArgsCount = function(name) {
      var checker;
      checker = this.checkers.get(name);
      return (checker != null ? checker.length : void 0) || 0;
    };

    ArgumentsValidator.prototype.splitArguments = function(name, oargs) {
      var args, nargs, restArgs;
      nargs = this.getArgsCount(name);
      args = _.slice(oargs, 0, nargs);
      restArgs = _.slice(oargs, nargs);
      return {
        args: args,
        restArgs: restArgs
      };
    };

    ArgumentsValidator.prototype.checkMessage = function(msg) {
      return check.object(msg) && check.string(msg.textMessage) && _.keys(msg).length === 1;
    };

    ArgumentsValidator.prototype.checkObject = function(obj) {
      return check.object(obj);
    };

    ArgumentsValidator.prototype.checkTypes = function(checkers, args) {
      var checker, i, idx, len;
      if ((args != null ? args.length : void 0) !== checkers.length) {
        return new ChatServiceError('wrongArgumentsCount', checkers.length, args.length);
      }
      for (idx = i = 0, len = checkers.length; i < len; idx = ++i) {
        checker = checkers[idx];
        if (!checker(args[idx])) {
          return new ChatServiceError('badArgument', idx, args[idx]);
        }
      }
      return null;
    };

    ArgumentsValidator.prototype.directAddToList = function(listName, userNames) {
      return [check.string, check.array.of.string];
    };

    ArgumentsValidator.prototype.directGetAccessList = function(listName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.directGetWhitelistMode = function() {
      return [];
    };

    ArgumentsValidator.prototype.directMessage = function(toUser, msg) {
      return [check.string, this.directMessagesChecker ? this.checkObject : this.checkMessage];
    };

    ArgumentsValidator.prototype.directRemoveFromList = function(listName, userNames) {
      return [check.string, check.array.of.string];
    };

    ArgumentsValidator.prototype.directSetWhitelistMode = function(mode) {
      return [check.boolean];
    };

    ArgumentsValidator.prototype.disconnect = function(reason) {
      return [check.string];
    };

    ArgumentsValidator.prototype.listOwnSockets = function() {
      return [];
    };

    ArgumentsValidator.prototype.roomAddToList = function(roomName, listName, userNames) {
      return [check.string, check.string, check.array.of.string];
    };

    ArgumentsValidator.prototype.roomCreate = function(roomName, mode) {
      return [check.string, check.boolean];
    };

    ArgumentsValidator.prototype.roomDelete = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomGetAccessList = function(roomName, listName) {
      return [check.string, check.string];
    };

    ArgumentsValidator.prototype.roomGetOwner = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomGetWhitelistMode = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomRecentHistory = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomHistoryGet = function(roomName, id, limit) {
      return [
        check.string, function(str) {
          return check.greaterOrEqual(str, 0);
        }, function(str) {
          return check.greaterOrEqual(str, 1);
        }
      ];
    };

    ArgumentsValidator.prototype.roomHistoryInfo = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomJoin = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomLeave = function(roomName) {
      return [check.string];
    };

    ArgumentsValidator.prototype.roomMessage = function(roomName, msg) {
      return [check.string, this.roomMessagesChecker ? this.checkObject : this.checkMessage];
    };

    ArgumentsValidator.prototype.roomRemoveFromList = function(roomName, listName, userNames) {
      return [check.string, check.string, check.array.of.string];
    };

    ArgumentsValidator.prototype.roomSetWhitelistMode = function(roomName, mode) {
      return [check.string, check.boolean];
    };

    ArgumentsValidator.prototype.roomUserSeen = function(roomName, userName) {
      return [check.string, check.string];
    };

    ArgumentsValidator.prototype.systemMessage = function(data) {
      return [
        function() {
          return true;
        }
      ];
    };

    return ArgumentsValidator;

  })();

  module.exports = ArgumentsValidator;

}).call(this);

//# sourceMappingURL=ArgumentsValidator.js.map
