// Generated by CoffeeScript 1.10.0
(function() {
  var ChatServiceError, DirectMessagingStateRedis, ListsStateRedis, Promise, Redis, RedisState, Room, RoomStateRedis, User, UserStateRedis, _, extend, initSet, lockOperations, luaCommands, namespace, promiseRetry, stateOperations, uid,
    extend1 = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty;

  ChatServiceError = require('./ChatServiceError');

  Promise = require('bluebird');

  Redis = require('ioredis');

  Room = require('./Room');

  User = require('./User');

  _ = require('lodash');

  promiseRetry = require('promise-retry');

  uid = require('uid-safe');

  extend = require('./utils').extend;

  namespace = 'chatservice';

  initSet = function(redis, set, values) {
    return redis.del(set).then(function() {
      if (!values) {
        return Promise.resolve();
      }
      return redis.sadd(set, values);
    });
  };

  stateOperations = {
    initState: function(state) {
      return this.redis.setnx(this.makeKeyName('exists'), true).then((function(_this) {
        return function(isnew) {
          var error;
          if (!isnew) {
            error = new ChatServiceError(_this.exitsErrorName, _this.name);
            return Promise.reject(error);
          }
        };
      })(this)).then((function(_this) {
        return function() {
          return _this.stateReset(state);
        };
      })(this)).then((function(_this) {
        return function() {
          return _this.redis.setnx(_this.makeKeyName('isInit'), true);
        };
      })(this));
    },
    removeState: function() {
      return this.stateReset(null).then((function(_this) {
        return function() {
          return _this.redis.del(_this.makeKeyName('exists'), _this.makeKeyName('isInit'));
        };
      })(this));
    },
    startRemoving: function() {
      return this.redis.del(this.makeKeyName('isInit'));
    }
  };

  lockOperations = {
    lock: function(key, val, ttl) {
      return promiseRetry({
        minTimeout: 100,
        retries: 10,
        factor: 1.5,
        randomize: true
      }, (function(_this) {
        return function(retry, n) {
          return _this.redis.set(key, val, 'NX', 'PX', ttl).then(function(res) {
            var err;
            if (!res) {
              err = new ChatServiceError('timeout');
              return retry(err);
            }
          })["catch"](retry);
        };
      })(this));
    },
    unlock: function(key, val) {
      return this.redis.unlock(key, val);
    }
  };

  luaCommands = {
    unlock: {
      numberOfKeys: 1,
      lua: "if redis.call(\"get\",KEYS[1]) == ARGV[1] then\n  return redis.call(\"del\",KEYS[1])\nelse\n  return 0\nend"
    },
    messageAdd: {
      numberOfKeys: 5,
      lua: "local msg = ARGV[1]\nlocal ts = ARGV[2]\n\nlocal lastMessageId = KEYS[1]\nlocal historyMaxSize = KEYS[2]\nlocal messagesIds = KEYS[3]\nlocal messagesTimestamps = KEYS[4]\nlocal messagesHistory = KEYS[5]\n\nlocal id = tonumber(redis.call('INCR', lastMessageId))\nlocal maxsz = tonumber(redis.call('GET', historyMaxSize))\n\nredis.call('LPUSH', messagesIds, id)\nredis.call('LPUSH', messagesTimestamps, ts)\nredis.call('LPUSH', messagesHistory, msg)\n\nlocal sz = tonumber(redis.call('LLEN', messagesHistory))\n\nif sz > maxsz then\n  redis.call('RPOP', messagesIds)\n  redis.call('RPOP', messagesTimestamps)\n  redis.call('RPOP', messagesHistory)\nend\n\nreturn {id}"
    },
    messagesGet: {
      numberOfKeys: 5,
      lua: "local id = ARGV[1]\nlocal maxlen = ARGV[2]\n\nlocal lastMessageId = KEYS[1]\nlocal historyMaxSize = KEYS[2]\nlocal messagesIds = KEYS[3]\nlocal messagesTimestamps = KEYS[4]\nlocal messagesHistory = KEYS[5]\n\nlocal lastid = tonumber(redis.call('GET', lastMessageId))\nlocal maxsz = tonumber(redis.call('GET', historyMaxSize))\nlocal id = math.min(id, lastid)\nlocal endp = lastid - id\nlocal len = math.min(maxlen, endp)\nlocal start = math.max(0, endp - len)\n\nif start >= endp then\n  return {}\nend\n\nendp = endp - 1\nlocal msgs = redis.call('LRANGE', messagesHistory, start, endp)\nlocal tss = redis.call('LRANGE', messagesTimestamps, start, endp)\nlocal ids = redis.call('LRANGE', messagesIds, start, endp)\n\nreturn {msgs, tss, ids}"
    },
    getSocketsToRooms: {
      numberOfKeys: 1,
      lua: "local result = {}\nlocal sockets = KEYS[1]\nlocal prefix = ARGV[1]\nlocal ids = redis.call('HKEYS', sockets)\n\nif table.getn(ids) == 0 then\n  local jsonResult = cjson.encode(cjson.null)\n  return {jsonResult}\nend\n\nfor i, id in pairs(ids) do\n  local joined = redis.call('SMEMBERS', prefix .. id)\n  result[id] = joined\nend\n\nlocal jsonResult = cjson.encode(result)\nreturn {jsonResult}"
    },
    removeAllSocketsFromRoom: {
      numberOfKeys: 1,
      lua: "local room = KEYS[1]\nlocal prefix = ARGV[1]\nlocal roomName = ARGV[2]\nlocal ids = redis.call('SMEMBERS', room)\n\nif table.getn(ids) == 0 then\n  local jsonResult = cjson.encode(cjson.null)\n  return {jsonResult}\nend\n\nredis.call('DEL', room)\n\nfor i, id in pairs(ids) do\n  redis.call('SREM', prefix .. id, roomName)\nend\n\nlocal jsonResult = cjson.encode(ids)\nreturn {jsonResult}"
    },
    removeSocket: {
      numberOfKeys: 2,
      lua: "local id = KEYS[1]\nlocal sockets = KEYS[2]\nlocal prefix = ARGV[1]\nlocal socketid = ARGV[2]\n\nlocal rooms = redis.call('SMEMBERS', id)\nredis.call('DEL', id)\n\nredis.call('HDEL', sockets, socketid)\nlocal nconnected = redis.call('HLEN', sockets)\n\nlocal removedRooms = {}\nlocal joinedSockets = {}\n\nfor i, room in pairs(rooms) do\n  local ismember = redis.call('SISMEMBER', prefix .. room, socketid)\n  if ismember == 1 then\n    redis.call('SREM', prefix .. room, socketid)\n    local njoined = redis.call('SCARD', prefix .. room)\n    table.insert(removedRooms, room)\n    table.insert(joinedSockets, njoined)\n  end\nend\n\nif table.getn(removedRooms) == 0 or table.getn(rooms) == 0 then\n  local jsonResult = cjson.encode({cjson.null, cjson.null, nconnected})\n  return {jsonResult}\nend\n\nlocal jsonResult = cjson.encode({removedRooms, joinedSockets, nconnected})\nreturn {jsonResult}"
    }
  };

  ListsStateRedis = (function() {
    function ListsStateRedis() {}

    ListsStateRedis.prototype.makeKeyName = function(keyName) {
      return namespace + ":" + this.prefix + ":{" + this.name + "}:" + keyName;
    };

    ListsStateRedis.prototype.checkList = function(listName) {
      var error;
      if (!this.hasList(listName)) {
        error = new ChatServiceError('noList', listName);
        return Promise.reject(error);
      }
      return Promise.resolve();
    };

    ListsStateRedis.prototype.addToList = function(listName, elems) {
      return this.checkList(listName).then((function(_this) {
        return function() {
          return _this.redis.sadd(_this.makeKeyName(listName), elems);
        };
      })(this));
    };

    ListsStateRedis.prototype.removeFromList = function(listName, elems) {
      return this.checkList(listName).then((function(_this) {
        return function() {
          return _this.redis.srem(_this.makeKeyName(listName), elems);
        };
      })(this));
    };

    ListsStateRedis.prototype.getList = function(listName) {
      return this.checkList(listName).then((function(_this) {
        return function() {
          return _this.redis.smembers(_this.makeKeyName(listName));
        };
      })(this));
    };

    ListsStateRedis.prototype.hasInList = function(listName, elem) {
      return this.checkList(listName).then((function(_this) {
        return function() {
          return _this.redis.sismember(_this.makeKeyName(listName), elem);
        };
      })(this)).then(function(data) {
        var result;
        result = data ? true : false;
        return Promise.resolve(result);
      });
    };

    ListsStateRedis.prototype.whitelistOnlySet = function(mode) {
      var whitelistOnly;
      whitelistOnly = mode ? true : '';
      return this.redis.set(this.makeKeyName('whitelistMode'), whitelistOnly);
    };

    ListsStateRedis.prototype.whitelistOnlyGet = function() {
      return this.redis.get(this.makeKeyName('whitelistMode')).then(function(data) {
        var result;
        result = data ? true : false;
        return Promise.resolve(result);
      });
    };

    return ListsStateRedis;

  })();

  RoomStateRedis = (function(superClass) {
    extend1(RoomStateRedis, superClass);

    extend(RoomStateRedis, stateOperations);

    function RoomStateRedis(server, roomName1) {
      this.server = server;
      this.roomName = roomName1;
      this.name = this.roomName;
      this.historyMaxGetMessages = this.server.historyMaxGetMessages;
      this.redis = this.server.redis;
      this.exitsErrorName = 'roomExists';
      this.prefix = 'rooms';
    }

    RoomStateRedis.prototype.stateReset = function(state) {
      var adminlist, blacklist, historyMaxSize, owner, whitelist, whitelistOnly;
      if (state == null) {
        state = {};
      }
      whitelist = state.whitelist, blacklist = state.blacklist, adminlist = state.adminlist, whitelistOnly = state.whitelistOnly, owner = state.owner, historyMaxSize = state.historyMaxSize;
      whitelistOnly = whitelistOnly ? true : '';
      if (!owner) {
        owner = '';
      }
      return Promise.all([initSet(this.redis, this.makeKeyName('whitelist'), whitelist), initSet(this.redis, this.makeKeyName('blacklist'), blacklist), initSet(this.redis, this.makeKeyName('adminlist'), adminlist), initSet(this.redis, this.makeKeyName('userlist'), null), this.redis.del(this.makeKeyName('messagesHistory')), this.redis.del(this.makeKeyName('messagesTimestamps')), this.redis.del(this.makeKeyName('messagesIds')), this.redis.del(this.makeKeyName('usersseen')), this.redis.set(this.makeKeyName('lastMessageId'), 0), this.redis.set(this.makeKeyName('whitelistMode'), whitelistOnly), this.redis.set(this.makeKeyName('owner'), owner), this.historyMaxSizeSet(historyMaxSize)])["return"]();
    };

    RoomStateRedis.prototype.hasList = function(listName) {
      return listName === 'adminlist' || listName === 'whitelist' || listName === 'blacklist' || listName === 'userlist';
    };

    RoomStateRedis.prototype.ownerGet = function() {
      return this.redis.get(this.makeKeyName('owner'));
    };

    RoomStateRedis.prototype.ownerSet = function(owner) {
      return this.redis.set(this.makeKeyName('owner'), owner);
    };

    RoomStateRedis.prototype.historyMaxSizeSet = function(historyMaxSize) {
      if (_.isNumber(historyMaxSize) && historyMaxSize >= 0) {
        return this.redis.set(this.makeKeyName('historyMaxSize'), historyMaxSize);
      } else {
        return this.redis.set(this.makeKeyName('historyMaxSize'), this.server.defaultHistoryLimit);
      }
    };

    RoomStateRedis.prototype.historyInfo = function() {
      return this.redis.multi().get(this.makeKeyName('historyMaxSize')).llen(this.makeKeyName('messagesHistory')).get(this.makeKeyName('lastMessageId')).exec().spread((function(_this) {
        return function(arg, arg1, arg2) {
          var _0, _1, _2, historyMaxSize, historySize, info, lastMessageId;
          _0 = arg[0], historyMaxSize = arg[1];
          _1 = arg1[0], historySize = arg1[1];
          _2 = arg2[0], lastMessageId = arg2[1];
          historySize = parseInt(historySize);
          historyMaxSize = parseFloat(historyMaxSize);
          lastMessageId = parseInt(lastMessageId);
          info = {
            historySize: historySize,
            historyMaxSize: historyMaxSize,
            historyMaxGetMessages: _this.historyMaxGetMessages,
            lastMessageId: lastMessageId
          };
          return Promise.resolve(info);
        };
      })(this));
    };

    RoomStateRedis.prototype.getCommonUsers = function() {
      return this.redis.sdiff(this.makeKeyName('userlist'), this.makeKeyName('whitelist'), this.makeKeyName('adminlist'));
    };

    RoomStateRedis.prototype.messageAdd = function(msg) {
      var smsg, timestamp;
      timestamp = _.now();
      smsg = JSON.stringify(msg);
      return this.redis.messageAdd(this.makeKeyName('lastMessageId'), this.makeKeyName('historyMaxSize'), this.makeKeyName('messagesIds'), this.makeKeyName('messagesTimestamps'), this.makeKeyName('messagesHistory'), smsg, timestamp).spread(function(id) {
        msg.id = id;
        msg.timestamp = timestamp;
        return Promise.resolve(msg);
      });
    };

    RoomStateRedis.prototype.convertMessages = function(msgs, tss, ids) {
      var data, i, idx, len, msg, obj;
      data = [];
      if (!msgs) {
        return Promise.resolve(data);
      }
      for (idx = i = 0, len = msgs.length; i < len; idx = ++i) {
        msg = msgs[idx];
        obj = JSON.parse(msg, function(key, val) {
          if ((val != null ? val.type : void 0) === 'Buffer') {
            return new Buffer(val.data);
          } else {
            return val;
          }
        });
        obj.timestamp = parseInt(tss[idx]);
        obj.id = parseInt(ids[idx]);
        data[idx] = obj;
      }
      return Promise.resolve(data);
    };

    RoomStateRedis.prototype.messagesGetRecent = function() {
      if (this.historyMaxGetMessages <= 0) {
        return Promise.resolve([]);
      }
      return this.redis.multi().lrange(this.makeKeyName('messagesHistory'), 0, this.historyMaxGetMessages - 1).lrange(this.makeKeyName('messagesTimestamps'), 0, this.historyMaxGetMessages - 1).lrange(this.makeKeyName('messagesIds'), 0, this.historyMaxGetMessages - 1).exec().spread((function(_this) {
        return function(arg, arg1, arg2) {
          var _0, _1, _2, ids, msgs, tss;
          _0 = arg[0], msgs = arg[1];
          _1 = arg1[0], tss = arg1[1];
          _2 = arg2[0], ids = arg2[1];
          return _this.convertMessages(msgs, tss, ids);
        };
      })(this));
    };

    RoomStateRedis.prototype.messagesGet = function(id, maxMessages) {
      if (maxMessages == null) {
        maxMessages = this.historyMaxGetMessages;
      }
      if (maxMessages <= 0) {
        return Promise.resolve([]);
      }
      id = _.max([0, id]);
      return this.redis.messagesGet(this.makeKeyName('lastMessageId'), this.makeKeyName('historyMaxSize'), this.makeKeyName('messagesIds'), this.makeKeyName('messagesTimestamps'), this.makeKeyName('messagesHistory'), id, maxMessages).spread((function(_this) {
        return function(msgs, tss, ids) {
          return _this.convertMessages(msgs, tss, ids);
        };
      })(this));
    };

    RoomStateRedis.prototype.userSeenGet = function(userName) {
      return this.redis.multi().hget(this.makeKeyName('usersseen'), userName).sismember(this.makeKeyName('userlist'), userName).exec().spread(function(arg, arg1) {
        var _1, _2, isjoined, joined, timestamp, ts;
        _1 = arg[0], ts = arg[1];
        _2 = arg1[0], isjoined = arg1[1];
        joined = isjoined ? true : false;
        timestamp = ts ? parseInt(ts) : null;
        return {
          joined: joined,
          timestamp: timestamp
        };
      });
    };

    RoomStateRedis.prototype.userSeenUpdate = function(userName) {
      var timestamp;
      timestamp = _.now();
      return this.redis.hset(this.makeKeyName('usersseen'), userName, timestamp);
    };

    return RoomStateRedis;

  })(ListsStateRedis);

  DirectMessagingStateRedis = (function(superClass) {
    extend1(DirectMessagingStateRedis, superClass);

    extend(DirectMessagingStateRedis, stateOperations);

    function DirectMessagingStateRedis(server, userName1) {
      this.server = server;
      this.userName = userName1;
      this.name = this.userName;
      this.prefix = 'users';
      this.exitsErrorName = 'userExists';
      this.redis = this.server.redis;
    }

    DirectMessagingStateRedis.prototype.hasList = function(listName) {
      return listName === 'whitelist' || listName === 'blacklist';
    };

    DirectMessagingStateRedis.prototype.stateReset = function(state) {
      var blacklist, whitelist, whitelistOnly;
      if (state == null) {
        state = {};
      }
      whitelist = state.whitelist, blacklist = state.blacklist, whitelistOnly = state.whitelistOnly;
      whitelistOnly = whitelistOnly ? true : '';
      return Promise.all([initSet(this.redis, this.makeKeyName('whitelist'), whitelist), initSet(this.redis, this.makeKeyName('blacklist'), blacklist), this.redis.set(this.makeKeyName('whitelistMode'), whitelistOnly)])["return"]();
    };

    return DirectMessagingStateRedis;

  })(ListsStateRedis);

  UserStateRedis = (function() {
    extend(UserStateRedis, lockOperations);

    function UserStateRedis(server, userName1) {
      this.server = server;
      this.userName = userName1;
      this.name = this.userName;
      this.prefix = 'users';
      this.redis = this.server.redis;
      this.echoChannel = this.makeEchoChannelName(this.userName);
    }

    UserStateRedis.prototype.makeKeyName = function(keyName) {
      return namespace + ":" + this.prefix + ":{" + this.name + "}:" + keyName;
    };

    UserStateRedis.prototype.makeSocketToRooms = function(id) {
      if (id == null) {
        id = '';
      }
      return this.makeKeyName("socketsToRooms:" + id);
    };

    UserStateRedis.prototype.makeRoomToSockets = function(room) {
      if (room == null) {
        room = '';
      }
      return this.makeKeyName("roomsToSockets:" + room);
    };

    UserStateRedis.prototype.makeRoomLock = function(room) {
      if (room == null) {
        room = '';
      }
      return this.makeKeyName("roomLock:" + room);
    };

    UserStateRedis.prototype.makeEchoChannelName = function(userName) {
      return "echo:" + userName;
    };

    UserStateRedis.prototype.addSocket = function(id, uid) {
      return this.redis.multi().hset(this.makeKeyName('sockets'), id, uid).hlen(this.makeKeyName('sockets')).exec().spread(function(_0, arg) {
        var _1, nconnected;
        _1 = arg[0], nconnected = arg[1];
        return Promise.resolve(nconnected);
      });
    };

    UserStateRedis.prototype.getAllSockets = function() {
      return this.redis.hkeys(this.makeKeyName('sockets'));
    };

    UserStateRedis.prototype.getSocketsToInstance = function() {
      return this.redis.hgetall(this.makeKeyName('sockets'));
    };

    UserStateRedis.prototype.getRoomToSockets = function(roomName) {
      return this.redis.smembers(this.makeRoomToSockets(roomName));
    };

    UserStateRedis.prototype.getSocketsToRooms = function() {
      return this.redis.getSocketsToRooms(this.makeKeyName('sockets'), this.makeSocketToRooms()).spread(function(result) {
        var data, k, v;
        data = JSON.parse(result) || {};
        for (k in data) {
          v = data[k];
          if (_.isEmpty(v)) {
            data[k] = [];
          }
        }
        return Promise.resolve(data);
      });
    };

    UserStateRedis.prototype.addSocketToRoom = function(id, roomName) {
      return this.redis.multi().sadd(this.makeSocketToRooms(id), roomName).sadd(this.makeRoomToSockets(roomName), id).scard(this.makeRoomToSockets(roomName)).exec().spread(function(_0, _1, arg) {
        var _2, njoined;
        _2 = arg[0], njoined = arg[1];
        return Promise.resolve(njoined);
      });
    };

    UserStateRedis.prototype.removeSocketFromRoom = function(id, roomName) {
      return this.redis.multi().srem(this.makeSocketToRooms(id), roomName).srem(this.makeRoomToSockets(roomName), id).scard(this.makeRoomToSockets(roomName)).exec().spread(function(_0, _1, arg) {
        var _2, njoined;
        _2 = arg[0], njoined = arg[1];
        return Promise.resolve(njoined);
      });
    };

    UserStateRedis.prototype.removeAllSocketsFromRoom = function(roomName) {
      return this.redis.removeAllSocketsFromRoom(this.makeRoomToSockets(roomName), this.makeSocketToRooms(), roomName).spread(function(result) {
        return Promise.resolve(JSON.parse(result));
      });
    };

    UserStateRedis.prototype.removeSocket = function(id) {
      return this.redis.removeSocket(this.makeSocketToRooms(id), this.makeKeyName('sockets'), this.makeRoomToSockets(), id).spread(function(result) {
        return Promise.resolve(JSON.parse(result));
      });
    };

    UserStateRedis.prototype.lockToRoom = function(roomName, ttl) {
      return uid(18).then((function(_this) {
        return function(val) {
          var start;
          start = _.now();
          return _this.lock(_this.makeRoomLock(roomName), val, ttl).then(function() {
            return Promise.resolve().disposer(function() {
              if (start + ttl < _.now()) {
                _this.server.emit('lockTimeExceeded', val, {
                  userName: _this.userName,
                  roomName: roomName
                });
              }
              return _this.unlock(_this.makeRoomLock(roomName), val);
            });
          });
        };
      })(this));
    };

    return UserStateRedis;

  })();

  RedisState = (function() {
    function RedisState(server, options) {
      var cmd, def, redisOptions;
      this.server = server;
      this.options = options != null ? options : {};
      this.closed = false;
      if (this.options.useCluster) {
        this.redis = (function(func, args, ctor) {
          ctor.prototype = func.prototype;
          var child = new ctor, result = func.apply(child, args);
          return Object(result) === result ? result : child;
        })(Redis.Cluster, this.options.redisOptions, function(){});
      } else {
        redisOptions = _.castArray(this.options.redisOptions);
        this.redis = (function(func, args, ctor) {
          ctor.prototype = func.prototype;
          var child = new ctor, result = func.apply(child, args);
          return Object(result) === result ? result : child;
        })(Redis, redisOptions, function(){});
      }
      this.RoomState = RoomStateRedis;
      this.UserState = UserStateRedis;
      this.DirectMessagingState = DirectMessagingStateRedis;
      this.lockTTL = this.options.lockTTL || 10000;
      this.instanceUID = this.server.instanceUID;
      this.server.redis = this.redis;
      for (cmd in luaCommands) {
        def = luaCommands[cmd];
        this.redis.defineCommand(cmd, {
          numberOfKeys: def.numberOfKeys,
          lua: def.lua
        });
      }
    }

    RedisState.prototype.makeKeyName = function(prefix, name, keyName) {
      return namespace + ":" + prefix + ":{" + name + "}:" + keyName;
    };

    RedisState.prototype.hasRoom = function(name) {
      return this.redis.get(this.makeKeyName('rooms', name, 'isInit'));
    };

    RedisState.prototype.hasUser = function(name) {
      return this.redis.get(this.makeKeyName('users', name, 'isInit'));
    };

    RedisState.prototype.close = function() {
      this.closed = true;
      return this.redis.quit()["return"]();
    };

    RedisState.prototype.getRoom = function(name, isPredicate) {
      var room;
      if (isPredicate == null) {
        isPredicate = false;
      }
      room = new Room(this.server, name);
      return this.hasRoom(name).then(function(exists) {
        var error;
        if (!exists) {
          if (isPredicate) {
            return Promise.resolve(null);
          } else {
            error = new ChatServiceError('noRoom', name);
            return Promise.reject(error);
          }
        }
        return Promise.resolve(room);
      });
    };

    RedisState.prototype.addRoom = function(name, state) {
      var room;
      room = new Room(this.server, name);
      return room.initState(state)["return"](room);
    };

    RedisState.prototype.removeRoom = function(name) {
      return Promise.resolve();
    };

    RedisState.prototype.addSocket = function(id, userName) {
      return this.redis.hset(this.makeKeyName('instances', this.instanceUID, 'sockets'), id, userName);
    };

    RedisState.prototype.removeSocket = function(id) {
      return this.redis.hdel(this.makeKeyName('instances', this.instanceUID, 'sockets'), id);
    };

    RedisState.prototype.getInstanceSockets = function(uid) {
      if (uid == null) {
        uid = this.instanceUID;
      }
      return this.redis.hgetall(this.makeKeyName('instances', uid, 'sockets'));
    };

    RedisState.prototype.updateHeartbeat = function() {
      return this.redis.set(this.makeKeyName('instances', this.instanceUID, 'heartbeat'), _.now()).catchReturn();
    };

    RedisState.prototype.getInstanceHeartbeat = function(uid) {
      if (uid == null) {
        uid = this.instanceUID;
      }
      return this.redis.get(this.makeKeyName('instances', uid, 'heartbeat')).then(function(ts) {
        if (ts) {
          return parseInt(ts);
        } else {
          return null;
        }
      });
    };

    RedisState.prototype.getOrAddUser = function(name, state) {
      var user;
      user = new User(this.server, name);
      return this.hasUser(name).then(function(exists) {
        if (!exists) {
          return user.initState(state);
        }
      })["catch"](ChatServiceError, function(e) {
        return user;
      })["return"](user);
    };

    RedisState.prototype.getUser = function(name, isPredicate) {
      var user;
      if (isPredicate == null) {
        isPredicate = false;
      }
      user = new User(this.server, name);
      return this.hasUser(name).then(function(exists) {
        var error;
        if (!exists) {
          if (isPredicate) {
            return Promise.resolve(null);
          } else {
            error = new ChatServiceError('noUser', name);
            return Promise.reject(error);
          }
        }
        return Promise.resolve(user);
      });
    };

    RedisState.prototype.addUser = function(name, state) {
      var user;
      user = new User(this.server, name);
      return user.initState(state)["return"](user);
    };

    RedisState.prototype.removeUser = function(name) {
      return Promise.resolve();
    };

    return RedisState;

  })();

  module.exports = RedisState;

}).call(this);

//# sourceMappingURL=RedisState.js.map
