// Generated by CoffeeScript 1.10.0
(function() {
  var ChatServiceError, Promise, Room, RoomPermissions, _, asyncLimit, extend, ref;

  ChatServiceError = require('./ChatServiceError');

  Promise = require('bluebird');

  _ = require('lodash');

  ref = require('./utils'), extend = ref.extend, asyncLimit = ref.asyncLimit;

  RoomPermissions = {
    isAdmin: function(userName) {
      return this.roomState.ownerGet().then((function(_this) {
        return function(owner) {
          if (owner === userName) {
            return true;
          }
          return _this.roomState.hasInList('adminlist', userName);
        };
      })(this));
    },
    hasRemoveChangedCurrentAccess: function(userName, listName) {
      return this.roomState.hasInList('userlist', userName).then((function(_this) {
        return function(hasUser) {
          if (!hasUser) {
            return false;
          }
          return _this.isAdmin(userName).then(function(admin) {
            if (admin || listName !== 'whitelist') {
              return false;
            } else {
              return _this.roomState.whitelistOnlyGet();
            }
          });
        };
      })(this))["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            userName: userName
          });
        };
      })(this));
    },
    hasAddChangedCurrentAccess: function(userName, listName) {
      return this.roomState.hasInList('userlist', userName).then((function(_this) {
        return function(hasUser) {
          if (!hasUser) {
            return false;
          }
          return _this.isAdmin(userName).then(function(admin) {
            if (admin || listName !== 'blacklist') {
              return false;
            } else {
              return true;
            }
          });
        };
      })(this))["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            userName: userName
          });
        };
      })(this));
    },
    getModeChangedCurrentAccess: function(value) {
      if (!value) {
        return [];
      } else {
        return this.roomState.getCommonUsers();
      }
    },
    checkListChanges: function(author, listName, values, bypassPermissions) {
      return this.roomState.ownerGet().then((function(_this) {
        return function(owner) {
          if (listName === 'userlist') {
            return Promise.reject(new ChatServiceError('notAllowed'));
          }
          if (author === owner || bypassPermissions) {
            return;
          }
          if (listName === 'adminlist') {
            return Promise.reject(new ChatServiceError('notAllowed'));
          }
          return _this.roomState.hasInList('adminlist', author).then(function(admin) {
            var i, len, name;
            if (!admin) {
              return Promise.reject(new ChatServiceError('notAllowed'));
            }
            for (i = 0, len = values.length; i < len; i++) {
              name = values[i];
              if (name === owner) {
                return Promise.reject(new ChatServiceError('notAllowed'));
              }
            }
          });
        };
      })(this));
    },
    checkModeChange: function(author, value, bypassPermissions) {
      return this.isAdmin(author).then(function(admin) {
        if (!(admin || bypassPermissions)) {
          return Promise.reject(new ChatServiceError('notAllowed'));
        }
      });
    },
    checkAcess: function(userName) {
      return this.isAdmin(userName).then((function(_this) {
        return function(admin) {
          if (admin) {
            return;
          }
          return _this.roomState.hasInList('blacklist', userName).then(function(blacklisted) {
            if (blacklisted) {
              return Promise.reject(new ChatServiceError('notAllowed'));
            }
            return _this.roomState.whitelistOnlyGet().then(function(whitelistOnly) {
              if (!whitelistOnly) {
                return;
              }
              return _this.roomState.hasInList('whitelist', userName).then(function(whitelisted) {
                if (!whitelisted) {
                  return Promise.reject(new ChatServiceError('notAllowed'));
                }
              });
            });
          });
        };
      })(this));
    },
    checkRead: function(author, bypassPermissions) {
      if (bypassPermissions) {
        return Promise.resolve();
      }
      return this.isAdmin(author).then((function(_this) {
        return function(admin) {
          if (admin) {
            return;
          }
          return _this.roomState.hasInList('userlist', author).then(function(hasAuthor) {
            if (!hasAuthor) {
              return Promise.reject(new ChatServiceError('notJoined', _this.roomName));
            }
          });
        };
      })(this));
    },
    checkIsOwner: function(author, bypassPermissions) {
      if (bypassPermissions) {
        return Promise.resolve();
      }
      return this.roomState.ownerGet().then(function(owner) {
        if (owner === author) {
          return;
        }
        return Promise.reject(new ChatServiceError('notAllowed'));
      });
    }
  };

  Room = (function() {
    extend(Room, RoomPermissions);

    function Room(server, roomName) {
      var State;
      this.server = server;
      this.roomName = roomName;
      State = this.server.state.RoomState;
      this.roomState = new State(this.server, this.roomName);
    }

    Room.prototype.initState = function(state) {
      return this.roomState.initState(state);
    };

    Room.prototype.removeState = function() {
      return this.roomState.removeState();
    };

    Room.prototype.startRemoving = function() {
      return this.roomState.startRemoving();
    };

    Room.prototype.consistencyFailure = function(error, operationInfo) {
      if (operationInfo == null) {
        operationInfo = {};
      }
      operationInfo.roomName = this.roomName;
      operationInfo.opType = 'roomUserlist';
      this.server.emit('storeConsistencyFailure', error, operationInfo);
    };

    Room.prototype.getUsers = function() {
      return this.roomState.getList('userlist');
    };

    Room.prototype.leave = function(author) {
      return this.roomState.hasInList('userlist', author).then((function(_this) {
        return function(hasAuthor) {
          if (hasAuthor) {
            return _this.roomState.removeFromList('userlist', [author]).then(function() {
              return _this.roomState.userSeenUpdate(author);
            });
          }
        };
      })(this));
    };

    Room.prototype.join = function(author) {
      return this.checkAcess(author).then((function(_this) {
        return function() {
          return _this.roomState.hasInList('userlist', author);
        };
      })(this)).then((function(_this) {
        return function(hasAuthor) {
          if (!hasAuthor) {
            return _this.roomState.userSeenUpdate(author).then(function() {
              return _this.roomState.addToList('userlist', [author]);
            });
          }
        };
      })(this));
    };

    Room.prototype.message = function(author, msg, bypassPermissions) {
      return Promise["try"]((function(_this) {
        return function() {
          if (!bypassPermissions) {
            return _this.roomState.hasInList('userlist', author).then(function(hasAuthor) {
              if (!hasAuthor) {
                return Promise.reject(new ChatServiceError('notJoined', _this.roomName));
              }
            });
          }
        };
      })(this)).then((function(_this) {
        return function() {
          return _this.roomState.messageAdd(msg);
        };
      })(this));
    };

    Room.prototype.getList = function(author, listName, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.getList(listName);
        };
      })(this));
    };

    Room.prototype.getRecentMessages = function(author, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.messagesGetRecent();
        };
      })(this));
    };

    Room.prototype.getHistoryInfo = function(author, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.historyInfo();
        };
      })(this));
    };

    Room.prototype.getMessages = function(author, id, limit, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          if (!bypassPermissions) {
            limit = _.min([limit, _this.server.historyMaxGetMessages]);
          }
          return _this.roomState.messagesGet(id, limit);
        };
      })(this));
    };

    Room.prototype.addToList = function(author, listName, values, bypassPermissions) {
      return this.checkListChanges(author, listName, values, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.addToList(listName, values);
        };
      })(this)).then((function(_this) {
        return function() {
          return Promise.filter(values, function(val) {
            return _this.hasAddChangedCurrentAccess(val, listName);
          }, {
            concurrency: asyncLimit
          });
        };
      })(this));
    };

    Room.prototype.removeFromList = function(author, listName, values, bypassPermissions) {
      return this.checkListChanges(author, listName, values, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.removeFromList(listName, values);
        };
      })(this)).then((function(_this) {
        return function() {
          return Promise.filter(values, function(val) {
            return _this.hasRemoveChangedCurrentAccess(val, listName);
          }, {
            concurrency: asyncLimit
          });
        };
      })(this));
    };

    Room.prototype.getMode = function(author, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.whitelistOnlyGet();
        };
      })(this));
    };

    Room.prototype.getOwner = function(author, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.ownerGet();
        };
      })(this));
    };

    Room.prototype.changeMode = function(author, mode, bypassPermissions) {
      var whitelistOnly;
      whitelistOnly = mode;
      return this.checkModeChange(author, mode, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.whitelistOnlySet(whitelistOnly);
        };
      })(this)).then((function(_this) {
        return function() {
          return _this.getModeChangedCurrentAccess(whitelistOnly);
        };
      })(this)).then(function(usernames) {
        return [usernames, whitelistOnly];
      });
    };

    Room.prototype.userSeen = function(author, userName, bypassPermissions) {
      return this.checkRead(author, bypassPermissions).then((function(_this) {
        return function() {
          return _this.roomState.userSeenGet(userName);
        };
      })(this));
    };

    return Room;

  })();

  module.exports = Room;

}).call(this);

//# sourceMappingURL=Room.js.map
