// Generated by CoffeeScript 1.10.0
(function() {
  var ChatServiceError, CommandBinder, DirectMessaging, Promise, User, UserAssociations, _, asyncLimit, checkNameSymbols, extend, ref,
    extend1 = function(child, parent) { for (var key in parent) { if (hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; },
    hasProp = {}.hasOwnProperty,
    slice = [].slice;

  ChatServiceError = require('./ChatServiceError');

  CommandBinder = require('./CommandBinder');

  DirectMessaging = require('./DirectMessaging');

  Promise = require('bluebird');

  UserAssociations = require('./UserAssociations');

  _ = require('lodash');

  ref = require('./utils'), asyncLimit = ref.asyncLimit, checkNameSymbols = ref.checkNameSymbols, extend = ref.extend;

  User = (function(superClass) {
    extend1(User, superClass);

    extend(User, CommandBinder, UserAssociations);

    function User(server, userName1) {
      var State;
      this.server = server;
      this.userName = userName1;
      User.__super__.constructor.apply(this, arguments);
      this.state = this.server.state;
      this.transport = this.server.transport;
      this.validator = this.server.validator;
      this.hooks = this.server.hooks;
      this.enableUserlistUpdates = this.server.enableUserlistUpdates;
      this.enableAccessListsUpdates = this.server.enableAccessListsUpdates;
      this.enableRoomsManagement = this.server.enableRoomsManagement;
      this.enableDirectMessages = this.server.enableDirectMessages;
      State = this.server.state.UserState;
      this.userState = new State(this.server, this.userName);
      this.lockTTL = this.state.lockTTL;
      this.echoChannel = this.userState.echoChannel;
    }

    User.prototype.initState = function(state) {
      return User.__super__.initState.apply(this, arguments);
    };

    User.prototype.removeState = function() {
      return User.__super__.removeState.apply(this, arguments);
    };

    User.prototype.processMessage = function(msg, setTimestamp) {
      if (setTimestamp == null) {
        setTimestamp = false;
      }
      delete msg.id;
      delete msg.timestamp;
      if (setTimestamp) {
        msg.timestamp = _.now();
      }
      msg.author = this.userName || msg.author;
      return msg;
    };

    User.prototype.exec = function() {
      var args, cmd, command, error, fn, id, options;
      command = arguments[0], options = arguments[1], args = 3 <= arguments.length ? slice.call(arguments, 2) : [];
      if (options == null) {
        options = {};
      }
      id = options.id;
      if (!this.server.userCommands[command]) {
        error = new ChatServiceError('noCommand', command);
        return Promise.reject(error);
      }
      if (!id && (command === 'disconnect' || command === 'roomJoin' || command === 'roomLeave')) {
        error = new ChatServiceError('noSocket', command);
        return Promise.reject(error);
      }
      fn = this[command];
      cmd = this.makeCommand(command, fn);
      return Promise.fromCallback(function(cb) {
        return cmd.apply(null, slice.call(args).concat([options], [cb]));
      }, {
        multiArgs: true
      });
    };

    User.prototype.checkOnline = function() {
      return this.userState.getAllSockets().then((function(_this) {
        return function(sockets) {
          if (!(sockets != null ? sockets.length : void 0)) {
            return Promise.reject(new ChatServiceError('noUserOnline', _this.userName));
          }
        };
      })(this));
    };

    User.prototype.consistencyFailure = function(error, operationInfo) {
      var name;
      if (operationInfo == null) {
        operationInfo = {};
      }
      operationInfo.userName = this.userName;
      name = operationInfo.opType === 'transportChannel' ? 'transportConsistencyFailure' : 'storeConsistencyFailure';
      this.server.emit(name, error, operationInfo);
    };

    User.prototype.registerSocket = function(id) {
      return this.state.addSocket(id, this.userName).then((function(_this) {
        return function() {
          return _this.userState.addSocket(id, _this.server.instanceUID);
        };
      })(this)).then((function(_this) {
        return function(nconnected) {
          var cmd;
          if (!_this.transport.getConnectionObject(id)) {
            return _this.removeUserSocket(id).then(function() {
              return Promise.reject(new ChatServiceError('noSocket', 'connection'));
            });
          } else {
            for (cmd in _this.server.userCommands) {
              _this.bindCommand(id, cmd, _this[cmd]);
            }
            return [_this, nconnected];
          }
        };
      })(this));
    };

    User.prototype.disconnectInstanceSockets = function() {
      return this.userState.getAllSockets().then((function(_this) {
        return function(sockets) {
          return Promise.map(sockets, function(sid) {
            return _this.transport.disconnectClient(sid);
          }, {
            concurrency: asyncLimit
          });
        };
      })(this));
    };

    User.prototype.directAddToList = function(listName, values) {
      return this.addToList(this.userName, listName, values)["return"]();
    };

    User.prototype.directGetAccessList = function(listName) {
      return this.getList(this.userName, listName);
    };

    User.prototype.directGetWhitelistMode = function() {
      return this.getMode(this.userName);
    };

    User.prototype.directMessage = function(recipientName, msg, arg) {
      var bypassPermissions, error, id;
      id = arg.id, bypassPermissions = arg.bypassPermissions;
      if (!this.enableDirectMessages) {
        error = new ChatServiceError('notAllowed');
        return Promise.reject(error);
      }
      this.processMessage(msg, true);
      return this.server.state.getUser(recipientName).then((function(_this) {
        return function(recipient) {
          var channel;
          channel = recipient.echoChannel;
          return recipient.message(_this.userName, msg, bypassPermissions).then(function() {
            return recipient.checkOnline();
          }).then(function() {
            _this.transport.emitToChannel(channel, 'directMessage', msg);
            _this.transport.sendToChannel(id, _this.echoChannel, 'directMessageEcho', recipientName, msg);
            return msg;
          });
        };
      })(this));
    };

    User.prototype.directRemoveFromList = function(listName, values) {
      return this.removeFromList(this.userName, listName, values)["return"]();
    };

    User.prototype.directSetWhitelistMode = function(mode) {
      return this.changeMode(this.userName, mode)["return"]();
    };

    User.prototype.disconnect = function(reason, arg) {
      var id;
      id = arg.id;
      return this.removeSocketFromServer(id);
    };

    User.prototype.listOwnSockets = function() {
      return this.userState.getSocketsToRooms();
    };

    User.prototype.roomAddToList = function(roomName, listName, values, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.addToList(_this.userName, listName, values, bypassPermissions);
        };
      })(this)).then((function(_this) {
        return function(userNames) {
          if (_this.enableAccessListsUpdates) {
            _this.transport.emitToChannel(roomName, 'roomAccessListAdded', roomName, listName, values);
          }
          return _this.removeRoomUsers(roomName, userNames)["return"]();
        };
      })(this));
    };

    User.prototype.roomCreate = function(roomName, whitelistOnly, arg) {
      var bypassPermissions, error;
      bypassPermissions = arg.bypassPermissions;
      if (!this.enableRoomsManagement && !bypassPermissions) {
        error = new ChatServiceError('notAllowed');
        return Promise.reject(error);
      }
      return checkNameSymbols(roomName).then((function(_this) {
        return function() {
          return _this.state.addRoom(roomName, {
            owner: _this.userName,
            whitelistOnly: whitelistOnly
          });
        };
      })(this))["return"]();
    };

    User.prototype.roomDelete = function(roomName, arg) {
      var bypassPermissions, error;
      bypassPermissions = arg.bypassPermissions;
      if (!this.enableRoomsManagement && !bypassPermissions) {
        error = new ChatServiceError('notAllowed');
        return Promise.reject(error);
      }
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.checkIsOwner(_this.userName, bypassPermissions).then(function() {
            return room.startRemoving();
          }).then(function() {
            return room.getUsers();
          }).then(function(userNames) {
            return _this.removeRoomUsers(roomName, userNames);
          }).then(function() {
            return _this.state.removeRoom(roomName);
          }).then(function() {
            return room.removeState();
          })["return"]();
        };
      })(this));
    };

    User.prototype.roomGetAccessList = function(roomName, listName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getList(_this.userName, listName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomGetOwner = function(roomName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getOwner(_this.userName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomGetWhitelistMode = function(roomName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getMode(_this.userName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomRecentHistory = function(roomName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getRecentMessages(_this.userName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomHistoryGet = function(roomName, msgid, limit, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getMessages(_this.userName, msgid, limit, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomHistoryInfo = function(roomName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.getHistoryInfo(_this.userName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.roomJoin = function(roomName, arg) {
      var id;
      id = arg.id;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return _this.joinSocketToRoom(id, roomName);
        };
      })(this));
    };

    User.prototype.roomLeave = function(roomName, arg) {
      var id;
      id = arg.id;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return _this.leaveSocketFromRoom(id, room.roomName);
        };
      })(this));
    };

    User.prototype.roomMessage = function(roomName, msg, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          _this.processMessage(msg);
          return room.message(_this.userName, msg, bypassPermissions);
        };
      })(this)).then((function(_this) {
        return function(pmsg) {
          _this.transport.emitToChannel(roomName, 'roomMessage', roomName, pmsg);
          return pmsg.id;
        };
      })(this));
    };

    User.prototype.roomRemoveFromList = function(roomName, listName, values, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.removeFromList(_this.userName, listName, values, bypassPermissions);
        };
      })(this)).then((function(_this) {
        return function(userNames) {
          if (_this.enableAccessListsUpdates) {
            _this.transport.emitToChannel(roomName, 'roomAccessListRemoved', roomName, listName, values);
          }
          return _this.removeRoomUsers(roomName, userNames);
        };
      })(this))["return"]();
    };

    User.prototype.roomSetWhitelistMode = function(roomName, mode, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.changeMode(_this.userName, mode, bypassPermissions);
        };
      })(this)).spread((function(_this) {
        return function(userNames, mode) {
          if (_this.enableAccessListsUpdates) {
            _this.transport.emitToChannel(roomName, 'roomModeChanged', roomName, mode);
          }
          return _this.removeRoomUsers(roomName, userNames);
        };
      })(this));
    };

    User.prototype.roomUserSeen = function(roomName, userName, arg) {
      var bypassPermissions;
      bypassPermissions = arg.bypassPermissions;
      return this.state.getRoom(roomName).then((function(_this) {
        return function(room) {
          return room.userSeen(_this.userName, userName, bypassPermissions);
        };
      })(this));
    };

    User.prototype.systemMessage = function(data, arg) {
      var id;
      id = arg.id;
      this.transport.sendToChannel(id, this.echoChannel, 'systemMessage', data);
      return Promise.resolve();
    };

    return User;

  })(DirectMessaging);

  module.exports = User;

}).call(this);

//# sourceMappingURL=User.js.map
