// Generated by CoffeeScript 1.10.0
(function() {
  var Promise, UserAssociations, _, asyncLimit, eventToPromise;

  Promise = require('bluebird');

  _ = require('lodash');

  eventToPromise = require('event-to-promise');

  asyncLimit = require('./utils').asyncLimit;

  UserAssociations = {
    userJoinRoomReport: function(userName, roomName) {
      return this.transport.emitToChannel(roomName, 'roomUserJoined', roomName, userName);
    },
    userLeftRoomReport: function(userName, roomName) {
      return this.transport.emitToChannel(roomName, 'roomUserLeft', roomName, userName);
    },
    userRemovedReport: function(userName, roomName) {
      this.transport.emitToChannel(this.echoChannel, 'roomAccessRemoved', roomName);
      return this.userLeftRoomReport(userName, roomName);
    },
    socketJoinEcho: function(id, roomName, njoined) {
      return this.transport.sendToChannel(id, this.echoChannel, 'roomJoinedEcho', roomName, id, njoined);
    },
    socketLeftEcho: function(id, roomName, njoined) {
      return this.transport.sendToChannel(id, this.echoChannel, 'roomLeftEcho', roomName, id, njoined);
    },
    socketConnectEcho: function(id, nconnected) {
      return this.transport.sendToChannel(id, this.echoChannel, 'socketConnectEcho', id, nconnected);
    },
    socketDisconnectEcho: function(id, nconnected) {
      return this.transport.sendToChannel(id, this.echoChannel, 'socketDisconnectEcho', id, nconnected);
    },
    leaveChannel: function(id, channel) {
      return this.transport.leaveChannel(id, channel)["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            roomName: channel,
            id: id,
            opType: 'transportChannel'
          });
        };
      })(this));
    },
    socketLeaveChannels: function(id, channels) {
      return Promise.map(channels, (function(_this) {
        return function(channel) {
          return _this.leaveChannel(id, channel);
        };
      })(this), {
        concurrency: asyncLimit
      });
    },
    leaveChannelMessage: function(id, channel) {
      var bus;
      bus = this.transport.clusterBus;
      return Promise["try"](function() {
        bus.emit('roomLeaveSocket', id, channel);
        return eventToPromise(bus, bus.makeSocketRoomLeftName(id, channel));
      }).timeout(this.server.busAckTimeout).catchReturn();
    },
    channelLeaveSockets: function(channel, ids) {
      return Promise.map(ids, (function(_this) {
        return function(id) {
          return _this.leaveChannelMessage(id, channel);
        };
      })(this), {
        concurrency: asyncLimit
      });
    },
    rollbackRoomJoin: function(error, roomName, id) {
      return this.userState.removeSocketFromRoom(id, roomName)["catch"]((function(_this) {
        return function(e) {
          _this.consistencyFailure(e, {
            roomName: roomName,
            opType: 'userRooms'
          });
          return 1;
        };
      })(this)).then((function(_this) {
        return function(njoined) {
          if (!njoined) {
            return _this.leaveRoom(roomName);
          }
        };
      })(this)).thenThrow(error);
    },
    leaveRoom: function(roomName) {
      return Promise["try"]((function(_this) {
        return function() {
          return _this.state.getRoom(roomName);
        };
      })(this)).then((function(_this) {
        return function(room) {
          return room.leave(_this.userName);
        };
      })(this))["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            roomName: roomName,
            opType: 'roomUserlist'
          });
        };
      })(this));
    },
    joinSocketToRoom: function(id, roomName) {
      return Promise.using(this.userState.lockToRoom(roomName, this.lockTTL), (function(_this) {
        return function() {
          return _this.state.getRoom(roomName).then(function(room) {
            return room.join(_this.userName).then(function() {
              return _this.userState.addSocketToRoom(id, roomName).then(function(njoined) {
                return _this.transport.joinChannel(id, roomName).then(function() {
                  if (njoined === 1) {
                    _this.userJoinRoomReport(_this.userName, roomName);
                  }
                  return _this.socketJoinEcho(id, roomName, njoined);
                })["return"](njoined);
              })["catch"](function(e) {
                return _this.rollbackRoomJoin(e, roomName, id);
              });
            });
          });
        };
      })(this));
    },
    leaveSocketFromRoom: function(id, roomName) {
      return Promise.using(this.userState.lockToRoom(roomName, this.lockTTL), (function(_this) {
        return function() {
          return _this.userState.removeSocketFromRoom(id, roomName).then(function(njoined) {
            return _this.leaveChannel(id, roomName).then(function() {
              _this.socketLeftEcho(id, roomName, njoined);
              if (!njoined) {
                return _this.leaveRoom(roomName).then(function() {
                  return _this.userLeftRoomReport(_this.userName, roomName);
                });
              }
            })["return"](njoined);
          });
        };
      })(this));
    },
    removeUserSocket: function(id) {
      return this.userState.removeSocket(id).spread((function(_this) {
        return function(roomsRemoved, joinedSockets, nconnected) {
          if (roomsRemoved == null) {
            roomsRemoved = [];
          }
          if (joinedSockets == null) {
            joinedSockets = [];
          }
          if (nconnected == null) {
            nconnected = 0;
          }
          return _this.socketLeaveChannels(id, roomsRemoved).then(function() {
            return Promise.map(roomsRemoved, function(roomName, idx) {
              var njoined;
              njoined = joinedSockets[idx];
              _this.socketLeftEcho(id, roomName, njoined);
              if (!njoined) {
                return _this.leaveRoom(roomName).then(function() {
                  return _this.userLeftRoomReport(_this.userName, roomName);
                });
              }
            }, {
              concurrency: asyncLimit
            }).then(function() {
              return _this.socketDisconnectEcho(id, nconnected);
            });
          });
        };
      })(this)).then((function(_this) {
        return function() {
          return _this.state.removeSocket(id);
        };
      })(this));
    },
    removeSocketFromServer: function(id) {
      return this.removeUserSocket(id)["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            id: id,
            opType: 'userSockets'
          });
        };
      })(this));
    },
    removeUserSocketsFromRoom: function(roomName) {
      return this.userState.removeAllSocketsFromRoom(roomName)["catch"]((function(_this) {
        return function(e) {
          return _this.consistencyFailure(e, {
            roomName: roomName,
            opType: 'roomUserlist'
          });
        };
      })(this));
    },
    removeFromRoom: function(roomName) {
      return Promise.using(this.userState.lockToRoom(roomName, this.lockTTL), (function(_this) {
        return function() {
          return _this.removeUserSocketsFromRoom(roomName).then(function(removedSockets) {
            if (removedSockets == null) {
              removedSockets = [];
            }
            return _this.channelLeaveSockets(roomName, removedSockets).then(function() {
              if (removedSockets.length) {
                _this.userRemovedReport(_this.userName, roomName);
              }
              return _this.leaveRoom(roomName);
            });
          });
        };
      })(this));
    },
    removeRoomUsers: function(roomName, userNames) {
      if (userNames == null) {
        userNames = [];
      }
      return Promise.map(userNames, (function(_this) {
        return function(userName) {
          return _this.state.getUser(userName).then(function(user) {
            return user.removeFromRoom(roomName);
          }).catchReturn();
        };
      })(this), {
        concurrency: asyncLimit
      });
    }
  };

  module.exports = UserAssociations;

}).call(this);

//# sourceMappingURL=UserAssociations.js.map
